#ifndef PIEZO
#define PIEZO

//  we use the namespace for not conflict with other RingBuffer implementation 
namespace piezo {

    struct Value {
        int ms;
        int val;
    };


    /// @brief  Ringbuffer stores the Analog values of a piezo sensor together with the ms
    class RingBuffer {



    private:

        Value* _buf;





    protected:

        int  _freePos;         // first free cell
        int  _firstPos;        // first data cell
        int  _size;            // number of struct to be able to store    
        int  _msStart;         // Start of first cell in millis since cpu start
        int  _msDelta;         // delta in ms  between two values 
        bool _empty = true;    // the firs data cell has nothing
        int  _currentPos;      // for peek the next valeu
        bool _lastPos;         // in cse of Peek the last must be flaged to stop the reading


        void inc(int* x);
        void dec(int* x);


    public:
        RingBuffer() = default;

        RingBuffer(int size);

        bool FlVibStart;      // the first element is added to the RingBuffer, the idea is to start a timer for analyse the vibration 
        long MsVibLast;       // the last writte to the Buffert in millis of cpu time
        void Clear();         // reset the Ringbuffet
        bool Add(int x);      // add a value to the RingBuffet
        bool Get(Value* v);   // get a valu from the RingBuffer like Fifo
        bool Pop(Value* v);   // get a valu from the Ringbuffer like Lifo
        bool Peek(Value* v);  // reads the next value until last and next start again
    };



    class VibAnalyse {

    private:
        RingBuffer _ribu;

    public:
        byte nr;
        int lastValue = 0;
        VibAnalyse()  = default;

        VibAnalyse(int size, byte n);

        ~VibAnalyse();

        void RandomFillRingBuffer(int max); // for Testvalues

        bool NewSerie();
        void AddValue(int x, bool pct = false);
        int PeekSumVibration(     Value* v);
        int PeekVolumeVibration(  Value* v);
        int PeekPeakVibration(    Value* v);
        int PeekAverageVibration( Value* v);

        int GetSumVibration(      Value* v);
        int GetVolumeVibration(   Value* v);
        int GetPeakVibration(     Value* v);
        int GetAverageVibration(  Value* v);
    };


}
#endif // !PIEZO
